// (C) Copyright 1993,94 by Heiko Damerow MPG AG "Rntgenbeugung"
// Datei m_data.h
#ifndef __M_DATA_H
#define __M_DATA_H

typedef struct {
				int x0,x1,dx,y0,y1,dy;
				} TScreen;

typedef struct {
				double 		 	xMin,xMax,xSF;
				double 			yMin,yMax,ySF;
				double 			zMin,zMax,zSF;
				TScaleType  xST,yST,zST;
				} TCoorSystem;

typedef
	enum { ReciprokeLatticeBitmap = 1,MatrixBitmap,
				 Curve,Number,BarGraph,Histogram }
	TOutputType;

#define	DegToRad(val)  ((val) * 0.01745329251994)
#define	RadToDeg(val)  ((val) / 0.01745329251994)
#define ArToRLx(_ome,_the) ((cos(_ome) - cos((_the) - (_ome))))
#define ArToRLy(_ome,_the) ((sin(_ome) + sin((_the) - (_ome))))

class TBitmapSource
	{
	friend  class     TPlotData;
	friend  class     TCurveShowParam;

	public:           TBitmapSource(TPlotData *);
	virtual						~TBitmapSource();
	BOOL              ProcessBitmapFile(HFILE);
	void              FormatDBaseToBitmapSource(void);
	void              UpdateBitmapSource(void);
	BOOL              bGrayScalePalette;

	protected:
	void 							New(void);
	void              LoadBitmap(HDC);
	DWORD             GetImageSize(void);
	void              DrawBitmapFrame(HDC);
	void							SetScreen(HDC,TScreen&);
	void              DrawColorTable(HDC,TColorTable&);
	void              DrawMeasurementArea(HDC);
	void              FillBMInfoFromPalette(LOGPALETTE*);
	void              DrawBitmap(HDC,int,int);
	BITMAPFILEHEADER  BMPFileHdr;
	TColorTable       ColorTable;
	TPlotData         *DataWnd;

	private:
	HBITMAP           CreateGDIObject(HDC);
	HGLOBAL           RenderDIB(void);
	LOGPALETTE *      GetPalette(void);
	LOGPALETTE *      CreatePaletteFromDIB(void);
	LOGPALETTE *      CreateDefaultPalette(void);
	TOutputType       GetType(void);
	int               GetColor(float);
	void 							GenerateAngleSpaceBitmap(BOOL,int,int,DWORD);
	void 							GenerateRLBitmap(void);
	TCoorSystem*     	BMCoorSys;
	HGLOBAL           hDIBInfo;
	HGLOBAL						hDIBData;
	HGLOBAL						hMemPal;
	HPALETTE          hPalette,hOldPalette;
	HBITMAP           hBitmap;
	int               nEnlargedLines;
	int               nAddedPoints;
	};

class TPlotData : public TMDIWindow
	{
	// Stellt die Funktionalitt der Darstellung von Kurven und der Bewertung
	// dieser Kurven bereit
	friend  class  	TCurveShowParam;
	friend  class  	TBitmapSource;

	public:   			TPlotData(void);
	virtual					~TPlotData(void);
	void            Size(HWND,WPARAM,LPARAM);
	void            FormatCurveToPLine(TCurve *,LPPOINT,int &);
	void            Paint(HDC,PAINTSTRUCT *);
	virtual void    lButtonDown(WPARAM,LPARAM);
	virtual void    MouseMove(WPARAM,LPARAM);
	virtual void    lButtonUp(WPARAM,LPARAM);
	void            RenderFormat(WPARAM);
	void            RenderAllFormats(WPARAM);
	virtual	BOOL		New(void);
	void						PickUpData(LPARAM);
	void		        SetKSProperties(TKSProperties*);
	void            FreezeCurve(void);
	void            KillSecondCurve(void);
	void            SaveSecondCurve(void);
	void						EnablePaint(void) { bEnablePaint = TRUE;};
	void						DisablePaint(void) { bEnablePaint = FALSE;};

	protected:
	void            DrawCoordinateSystem(HDC);
	HCURSOR         GetCursor(void)  			{ return LoadCursor(NULL,IDC_CROSS);};
	void            DoCopy(void);
	virtual BOOL		SetMeasurementArea(TKSystem &);
	virtual void    SetRanges(void);
	virtual	double	GetThetaOffset(int) {return 0.0;};

	HFONT           hFont;
	TBitmapSource*  BSource;
	TCurve*         MainCurve;
	TCurve*         SecondCurve;
	BOOL            bSecondaryCoor;
	BOOL						bXY_Scaling;
	TOutputType     eOutputType;
	float           LowerBound,UpperBound;
	TCoorSystem     CoorSys,AltCoorSys;
  TScreen         Screen;

	private:
	BOOL						bEnablePaint;
	BOOL            bButtonDown;
	float           xo,yo;
	HDC             DragDC;
	};

class TCurveShowParam : public TModalDlg
	{
	public:           TCurveShowParam(TMDIWindow *);

	protected:
	void              Dlg_OnCommand(HWND,int,HWND,UINT);
	BOOL              Dlg_OnInit(HWND,HWND,LPARAM);
	BOOL              CanClose(void);
	BOOL              LeaveDialog(void);

	private:
	TPlotData*        PlotWindow;
	TCoorSystem*      CoorSys;
	TOutputType				bOutputToShow;
	BOOL							bBitmapIsCreated;
	HWND							hWndOutputT,hWndScaleT;
	};

#endif
// __M_DATA_H
// __LastLine__
