// (C) Copyright 1993,1994 by Heiko Damerow MPG AG "Rntgenbeugung"
// Datei m_scan.h
#ifndef __M_XSCAN_H
#define __M_XSCAN_H

#define CLIENTWINDOWSTYLE WS_CHILD|WS_CLIPCHILDREN|WS_VSCROLL|WS_HSCROLL

#include    "m_data.h"
#include		"m_psd.h"

const int   NoAskForSave 		= 0x9900;
const int   SaveWithNewName = 0x9901;
const int   nMaxScaleIdx    = 10;

class TScanParameters
	{
	public:         TScanParameters(void);
	int				      nMotor;
	TSaveFormat   	eSaveFormat;
	TxScanType      eScanType;
	BOOL          	bSaveOnReady;
	int							nCollectionAmount;
	double					dArgumentMin,dArgumentMax,dArgumentWidth;
	double					dSpeed;
	LPDevice      	Device;
	int           	nMFunctionId;     // Typ der Me-Funktion
	WORD          	nNumberCycle;     // Anzahl der Me-Zyklen
	float         	fFailure;         // Zur Konfiguration des Zhlers
	float         	fMaxTime;         // Maximale Mezeit
	DWORD         	dwMaxCounts;      // Maximale Anzahl Impulse
	LPDevice        Monitor;
	BOOL          	bSaveContinuous;
	BOOL          	bMonitorUsed;
	BOOL						bDynamicStepWidth;
	int							nStepScaleLevels;
	float						Scaler[20];
	};

class TScan : public TScanParameters, public TPlotData
	{
	friend class    TSetScanParameters;
	friend class    TSteering;
	friend class 		TScanCmd;
	friend class 		TSetupScanCmd;
	friend class 		TSaveDataCmd;

	public:         TScan(void);
									~TScan();

	protected:
	BOOL            LoadOldData(int);
	BOOL            SaveFile(int);
	BOOL            UpdateFile();
	BOOL            SaveDataBase(void);
	void            Create(HWND hwnd);
	void            CounterSetRequest(LPARAM);
	void            SteeringReady(LPARAM);
	void            SetRanges(void);
	void            DrawMenu(void);
	BOOL            New(void);
	BOOL            LoadMeasurementInfo(HFILE);
	BOOL            SaveMeasurementInfo(BOOL);
	void 						InitializeTask(WPARAM,LPARAM);
	void 						InitializeDlg(WPARAM,LPARAM);
	void            Interrupt();
	void            StopMeasurement();

	private:
	BOOL            bMacroRunning;
	BOOL            SetTitle(void);
	BOOL            NewFile();
	void            ExecuteNextCmd(void);
	BOOL            CanClose(void);
	void            rButtonDown(LPARAM);
	LPCSTR          ClassName() { return "Scan";};
	LPCSTR          GetCharacteristic() { return "Curve";};
	double          GetArgumentMin(void) {return dArgumentMin; };
	double          GetArgumentMax(void) {return dArgumentMax; };
	};

//******************************************************************************
class TAreaScanParameters
	{

	public:             TAreaScanParameters(void);

	int			            nOmega,nTheta;
	double  			      dOmegaMin,dOmegaMax,dOmegaWidth;
	double							dThetaMin,dThetaMax,dThetaWidth,dThetaOffset;
	double              dMoveRelation,dThetaWindow;
	TxScanType      		eScanType;
	TSaveFormat         eSaveFormat;
	BOOL                bSaveOnReady;
	BOOL                bSaveContinuous;
	BOOL								bPsdDevice;
	BOOL								bThetaAddsPsd;
	LPDevice            Device;
	BOOL                bAbsorberUsed;
	int									nAddedChannels;
	float               fFailure;          	// Zur Konfiguration des Zhlers
	float               fMaxTime;          	// Maximale Mezeit
	DWORD               dwMaxCounts;        // Maximale Anzahl Impulse
	LPDevice            Monitor;
	BOOL          			bMonitorUsed;
	TNotifyValue  			eNotifyValue;
	void								SetDevice(int);
	};

class TAreaScan : public TAreaScanParameters, public TPlotData
	{
	friend class    		TSteering;
	friend class 				TSetupScanCmd;
	friend class				TScanCmd;
	friend class 				TSetupAreaScanCmd;
	friend class				TAreaScanCmd;
	friend class 				TSaveDataCmd;
	friend class        TChooseScan;
	friend class        TAquisition;

	public: 						TAreaScan(void);
											~TAreaScan(void);

	protected:
	BOOL            		CanOpen(void);
	BOOL            		CanClose(void);
	BOOL           			LoadOldData(int);
	void            		Create(HWND);
	void            		DrawMenu(void);
	LRESULT             CallLocalAction(WPARAM,LPARAM);
	void            		rButtonDown(LPARAM);
	void 								lButtonUp(WPARAM,LPARAM);
	void 								lButtonDown(WPARAM,LPARAM);
	void 								InitializeTask(WPARAM,LPARAM);
	void 								InitializeDlg(WPARAM,LPARAM);
	void            		StopMeasurement(void);
	void            		Interrupt(void);
	void    						ExternSychronized(void);
	void            		CalibratePsd(void);
	BOOL								SetMeasurementArea(TKSystem &);
	BOOL            		New(void);
	BOOL            		SaveFile(int);
	BOOL            		SaveReport(void);
	BOOL            		LoadReport(void);
	void            		UpdateWnd(int);
	BOOL            		UpdateFile(void);
	void            		SetRanges(void);
	double							GetThetaOffset(int);
	LPCSTR  	      		ClassName() { return "AreaScan";};
	LPCSTR 	        		GetCharacteristic() { return "AreaScan";};
	void            		CounterSetRequest(LPARAM);
	void            		ShowSensorContinuous(BOOL);
	void            		SteeringReady(LPARAM);
	double           		GetArgumentMin(void) {return dOmegaMin; };
	double           		GetArgumentMax(void) {return dOmegaMax; };
	BOOL								bThetaScanCompleted;

	private:
	int									nActiveScan;
	HWND            		hOldControlWnd;
	BOOL            		bShowPsdContinuous;
	BOOL            		bCalibrationActive;
	BOOL            		bMacroRunning;
	BOOL            		bNewCurveNeeded;
	BOOL            		SetTitle(void);
	BOOL            		SaveMeasurementInfo(BOOL);
	BOOL            		LoadMeasurementInfo(HFILE);
	BOOL            		bFirstScan;
	LPCurve        		 	ScanReport;
	int 								GetAdditionalColumns(void);
	int									GetShift(int ScanNumber);
	};

//******************************************************************************
class TChooseScan : public TModelessDlg
	{
	public:							TChooseScan(TAreaScan*);

	private:
	BOOL                Dlg_OnInit(HWND,HWND,LPARAM);
	void                Dlg_OnCommand(HWND,int,HWND,UINT);
	void                Dlg_OnVScrollBar(HWND,HWND,UINT,int);
	void                Dlg_OnLButtonUp(HWND,int,int,UINT);
	BOOL 								CanClose(void);
	TAreaScan*          AScan;
	TCurve*             Curve;
  TCurve*             Report;
	HWND                BarHandle;
	int                 nMaxScan;
	};

class TSetupAreaScan : public TModalDlg
	{
	public:         TSetupAreaScan(TAreaScan*);

	private:
	int			        nOmega,nTheta,nOldMotor;
	LPDevice        Device;
	TAreaScan* 			Scan;
	HWND            hDeviceList;
	BOOL            CanClose(void);
	void            Dlg_OnCommand(HWND,int,HWND,UINT);
	BOOL            Dlg_OnInit(HWND,HWND,LPARAM);
	BOOL            LeaveDialog(void);
	};

class TAquisition : public TModalDlg
	{
	public:         TAquisition(void);

	private:
	int							nNumberPoints;
	int							nRange;
	char						FileName[20];
	HWND            hRangeList;
	BOOL            CanClose(void);
	void            Dlg_OnCommand(HWND,int,HWND,UINT);
	BOOL            Dlg_OnInit(HWND,HWND,LPARAM);
	BOOL            LeaveDialog(void);
	};

class TSetupContinuousScan : public TModalDlg
	{
	public:					TSetupContinuousScan(TScan*);

	private:
	HWND            hDeviceList,hMotorList;
	int			        nMotor,nOldMotor;
	TDevice*        Device;
	TScan* 					Scan;
	BOOL            CanClose(void);
	void            Dlg_OnCommand(HWND,int,HWND,UINT);
	BOOL            Dlg_OnInit(HWND,HWND,LPARAM);
	BOOL            LeaveDialog(void);
	double          dArgumentMin,dArgumentMax,dSpeed;
	int			        nMotorOld;
	LPDevice        DeviceOld;
	DWORD           dwMaxCounts;
	int							nCollectionAmount;
	float           fMaxTime;
	BOOL            bSaveOnReady;
	TSaveFormat     eSaveFormat;
	TxScanType      eScanType;
	};

class TSetupStepScan : public TModalDlg
	{
	public:							TSetupStepScan(TScan*);

	private:
	HWND            hDeviceList;
	HWND            hMotorList;
	HWND            hScanTypeList;
	int             nMotor,nOldMotor;
	LPDevice        Device,Monitor;
	TScan* 					Scan;
	BOOL            CanClose(void);
	void            Dlg_OnCommand(HWND,int,HWND,UINT);
	BOOL            Dlg_OnInit(HWND,HWND,LPARAM);
	BOOL            LeaveDialog(void) {return TRUE;};
	double          dArgumentMin,dArgumentMax,dArgumentWidth;
	int							nMotorOld;
	LPDevice        DeviceOld;
	LPDevice        MonitorOld;
	DWORD           dwMaxCounts;
	float           fMaxTime;
	BOOL            bMonitor,bSaveOnReady;
	TSaveFormat     eSaveFormat;
	TxScanType      eScanType;
	BOOL          	bSaveContinuous;
	BOOL          	bMonitorUsed;
	char            szPath[2*MaxString];
	};

class TCalibratePsd : public TModalDlg
	{
	public:					TCalibratePsd(TCurve&);
	int             GetBarEgde(int);
	int             GetBarPos(void);
	void            Dlg_OnHScrollBar(HWND,HWND,UINT,int);
	BOOL            Dlg_OnInit(HWND,HWND,LPARAM);
	void            Dlg_OnTimer(HWND,UINT);
	void            Dlg_OnCommand(HWND,int,HWND,UINT);
	BOOL            CanClose(void);
	BOOL            LeaveDialog(void);

	private:
	int             nAskTime;
	BOOL            bMoveActive;
	float           fRelationFactor;
	BOOL            bCalibrationOk;
	LPCurve					Curve;
	CPoint          LeftPeak;
	CPoint          RightPeak;
	BOOL            bGetRightPeak;
	BOOL            bGetLeftPeak;
	BOOL            bLongMove;
	float	          fOldPsdStep;
	double          dOldMotorWidth;
	HWND            hDlgItem;
	HWND            hBar;
	int             BarFactor;
	int							nMotor;
	long            dStepWidth;
	TDevice*				Device;
	};

class TPsdRemoteSync : public TModalDlg
	{
	public:					TPsdRemoteSync(TCurve&);
	BOOL            Dlg_OnInit(HWND,HWND,LPARAM);
	void            Dlg_OnTimer(HWND,UINT);
	void            Dlg_OnCommand(HWND,int,HWND,UINT);
	BOOL            CanClose(void);
	BOOL            LeaveDialog(void);

	private:
	void						PrintFileName(void);
	LPCSTR					GetFileName(void);
	int             nAskTime;
	BOOL            bCalibrationOk;
	LPCurve					Curve;
	float	          fOldPsdStep;
	HWND            hDlgItem;
	TDevice*				Device;
	int							nCurveCount;
	char						aPath[50];
	char						aName[14];
	float						fExposureTime;
	DWORD						dwExposureCounts;
	float						fError;
	char						aFileName[127];
	};

#endif
// __M_XSCAN_H
// __LastLine__
